/*
 * Copyright (C) The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package mobilevisionbitmapwrapper;

import anywheresoftware.b4a.AbsObjectWrapper;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.IOnActivityResult;
import anywheresoftware.b4a.BA.Permissions;

import com.google.android.gms.vision.Frame;
import com.google.android.gms.vision.text.Text;
import com.google.android.gms.vision.text.TextBlock;
import com.google.android.gms.vision.text.TextRecognizer;

import android.util.SparseArray;
import android.graphics.Bitmap;
import java.util.Calendar;

import android.widget.Toast;

import android.graphics.Matrix;


@ActivityObject
@ShortName("MobileVisionBitmap")
//@Events(values={"blocks_result(blocks As String)", "lines_result(lines As String)", "words_result(words As String)", "error_result(error As String)"})
//@Author("Github: Jean Carlos, Wrapped by: Johan Schoeman")
@Version(1.01f)
@DependsOn(values={"com.google.android.gms:play-services-vision"})
@Permissions(values={"android.permission.CAMERA", "android.permission.WRITE_EXTERNAL_STORAGE"})

public class mobilevisionbitmapWrapper {

	
	@Hide
	public static BA ba;
	@Hide
	public static String eventName;
	private TextRecognizer detector;
	private Bitmap bitmap;
	
	
	
	public void Initialize(BA paramBA, String paramString) {
	        eventName = paramString.toLowerCase(BA.cul);
	        ba = paramBA;
		
			//boolean initialvalues = getInitialValues();
			boolean initialvalues = true;
			if (initialvalues == true) {
              detector = new TextRecognizer.Builder(ba.applicationContext).build();
			} else {
				BA.Log("cannot start as expired");
				Toast.makeText(ba.applicationContext, "Trail period has expired - sorry!", Toast.LENGTH_LONG).show();
				ba.activity.finish();
			}			
		

	}	
	
    public void decodeBitmap (Bitmap bitmap) {
		    this.bitmap = bitmap;
            try {
                bitmap = getResizedBitmap(bitmap);
                if (detector.isOperational() && bitmap != null) {
					//BA.Log("detector is operational");
                    Frame frame = new Frame.Builder().setBitmap(bitmap).build();
                    SparseArray<TextBlock> textBlocks = detector.detect(frame);
                    String blocks = "";
                    String lines = "";
                    String words = "";
                    for (int index = 0; index < textBlocks.size(); index++) {
                        //extract scanned text blocks here
                        TextBlock tBlock = textBlocks.valueAt(index);
                        blocks = blocks + tBlock.getValue() + "\n" + "\n";
                        for (Text line : tBlock.getComponents()) {
                            //extract scanned text lines here
                            lines = lines + line.getValue() + "\n";
                            for (Text element : line.getComponents()) {
                                //extract scanned text words here
                                words = words + element.getValue() + ", ";

                            }
                        }
                    }
                    if (textBlocks.size() == 0) {
						if (ba.subExists(eventName + "_error_result")) {
							ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_error_result", true, new Object[] 
											{"Nothing Found!"});
						}			

                    } else {
						if (ba.subExists(eventName + "_blocks_result")) {
							ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_blocks_result", true, new Object[] 
											{blocks});
						}							
						
						if (ba.subExists(eventName + "_lines_result")) {
							ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_lines_result", true, new Object[] 
											{lines});
						}													
						
						if (ba.subExists(eventName + "_words_result")) {
							ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_words_result", true, new Object[] 
											{words});
						}							
                    }
                } else {
					if (ba.subExists(eventName + "_error_result")) {
						ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_error_result", true, new Object[] 
										{"Could not set up the detector!"});
					}									
					
                }
            } catch (Exception e) {
					if (ba.subExists(eventName + "_error_result")) {
						ba.raiseEventFromDifferentThread(ba.applicationContext, null, 0, eventName + "_error_result", true, new Object[] 
										{"Failed to load Image" + e.toString()});
					}									
            }		
		
	}	
	
	
	private Bitmap getResizedBitmap(Bitmap bm) {
		int width = bm.getWidth();
		int height = bm.getHeight();
		float scaleWidth = ((float) targetW) / width;
		float scaleHeight = ((float) targetH) / height;
		// CREATE A MATRIX FOR THE MANIPULATION
		Matrix matrix = new Matrix();
		// RESIZE THE BIT MAP
		matrix.postScale(scaleWidth, scaleHeight);

		// "RECREATE" THE NEW BITMAP
		Bitmap resizedBitmap = Bitmap.createBitmap(
			bm, 0, 0, width, height, matrix, false);
			
		bm.recycle();
		return resizedBitmap;
	}	


	private boolean getInitialValues() {
		Calendar c = Calendar.getInstance(); 
		int year = c.get(Calendar.YEAR);	
		int month = c.get(Calendar.MONTH);
		int day = c.get(Calendar.DAY_OF_MONTH);			
		
		if ((year == 2017) && (month == 9) && (day > 17) && (day <= 31)) {	
		   return true;
		} else {
		   return false;
		}  		
	}
	
	private int targetW = 1000;
	public void setTargetWidth(int targetW) {
		this.targetW = targetW;
	}
	
	private int targetH = 1000;
	public void setTargetHeight(int targetH) {
		this.targetH = targetH;
	}	
	
	

	
}
